# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Preisgarantietyp(StrEnum):
    """
    Aufzählung der Möglichkeiten für die Vergabe von Preisgarantien
    """

    ALLE_PREISBESTANDTEILE_BRUTTO = "ALLE_PREISBESTANDTEILE_BRUTTO"  #: ALLE_PREISBESTANDTEILE_BRUTTO
    ALLE_PREISBESTANDTEILE_NETTO = "ALLE_PREISBESTANDTEILE_NETTO"  #: ALLE_PREISBESTANDTEILE_NETTO
    PREISBESTANDTEILE_OHNE_ABGABEN = "PREISBESTANDTEILE_OHNE_ABGABEN"  #: PREISBESTANDTEILE_OHNE_ABGABEN
    NUR_ENERGIEPREIS = "NUR_ENERGIEPREIS"  #: NUR_ENERGIEPREIS
